module Manager.UpdateManager exposing (update)
-- Browser
import Browser
import Browser.Navigation as Nav
-- Http
import Http
-- Url
import Url
-- Json
import Json.Decode
-- Own
import TypeHolder exposing (Model, Msg(..))
import Manager.StateManager exposing (..)
import Manager.ModalManager exposing (..)
import Util.ShapeManager exposing (..)
import Util.Util         exposing (..)

-- UPDATE
update : Msg -> Model -> (Model, Cmd Msg)
update msg model =
   ( updateModel  msg model
   , updateCmdMsg msg model
   )
updateModel : Msg -> Model -> Model
updateModel msg model =
    case msg of
      ManageModal modalMsg value ->
        let modal = getModalFromState model.state
        in { model | state = manageState (ActiveModal
                                           (case modalMsg of
                                              CloseModal                 -> updateModal modalMsg                       modal
                                              OpenShowInput              -> updateModal modalMsg                       modal
                                              OpenShowTable              -> updateModal modalMsg                       modal
                                              OpenShowShape  _ _         -> updateModal modalMsg                       modal
                                              OpenShowJson               -> updateModal modalMsg                       modal
                                              OpenEditShape id ss iT _   -> updateModal (OpenEditShape id ss iT value) modal
                                              OpenManagePicture iT _     -> updateModal (OpenManagePicture iT value)   modal
                                              OpenLoadShapePicture _     -> updateModal (OpenLoadShapePicture value)   modal
                                              ResetShowInput             -> updateModal modalMsg                       modal
                                              Check          _           -> updateModal modalMsg                       modal
                                              Input          inputType _ -> updateModal (Input inputType value)        modal
                                              InputPoint                 -> updateModal modalMsg                       modal
                                           )
                                         )
           }
      ManageShapes shapesMsg value->
        case shapesMsg of
          DeleteShape _ -> if (eq_String_Modal "ShowTable" (getModalFromState model.state))
                             then { model | saved = updateShapes shapesMsg model.saved}
                             else { model | saved = updateShapes shapesMsg model.saved
                                          , state = Standby
                                  }
          AddShape    _ -> if (eq_String_Modal "ShowInput" (getModalFromState model.state))
                             then updateModel (ManageModal ResetShowInput "")
                                              { model | saved = updateShapes (AddShape (getInputShapeFromState model.state)) model.saved}
                             else { model | saved = updateShapes (AddShape (getInputShapeFromState model.state)) model.saved
                                          , state = Standby
                                  }
          SaveEditedShape _ _       -> { model | saved = updateShapes shapesMsg                       model.saved}
          SaveShapeStyle id sS iT _ -> { model | saved = updateShapes (SaveShapeStyle id sS iT value) model.saved}
      SvgShapeHover id     -> { model | svgShapeHover = id}
      OpenDrawShape shape  -> { model | state = DrawShape shape {x = 0, y = 0} }
      UpdateMousePos mouse -> case model.state of
          Standby         -> model -- ERROR
          ActiveModal _   -> model -- ERROR
          DrawShape iShape mOld -> { model | state = DrawShape iShape mouse }
      InputDrawShape -> case model.state of
          Standby         -> model -- ERROR
          ActiveModal _   -> model -- ERROR
          DrawShape iShape mPos -> case iShape of
              Empty             -> model -- ERROR
              Circle    x y r   ->
                if x=="" then { model | state = DrawShape (Circle (String.fromInt mPos.x)
                                                                  (String.fromInt mPos.y) r ) mPos }
                         else { model | saved = updateShapes (AddShape (Circle x y (abstand x y mPos))) model.saved
                                      , state = Standby
                              }
              Rectangle a b w h ->
                if a==""
                  then { model | state = DrawShape
                                           (Rectangle (String.fromInt mPos.x) (String.fromInt mPos.y) w h)
                                            mPos
                       }
                  else let coords = manageRectCoords
                                     {x= a
                                     ,y= b
                                     ,w= (abstand1D mPos.x (stringToInt a))
                                     ,h= (abstand1D mPos.y (stringToInt b))
                                     }
                       in { model | saved = updateShapes (AddShape (Rectangle coords.x coords.y coords.w coords.h)) model.saved
                                  , state = Standby
                          }
              Ellipse a b w h ->
                if a==""
                  then { model | state = DrawShape
                                           (Ellipse (String.fromInt mPos.x) (String.fromInt mPos.y) w h)
                                            mPos
                       }
                  else let coords = manageEllipseCoords
                                     {cx= a
                                     ,cy= b
                                     ,rx= (abstand1D mPos.x (stringToInt a))
                                     ,ry= (abstand1D mPos.y (stringToInt b))
                                     }
                       in { model | saved = updateShapes (AddShape (Ellipse coords.cx coords.cy coords.rx coords.ry)) model.saved
                                  , state = Standby
                          }
              Polygon   pList   -> { model | state = DrawShape
                                                      (Polygon (( String.fromInt mPos.x
                                                                , String.fromInt mPos.y
                                                                )::pList
                                                               )
                                                      )
                                                      mPos
                                   }
      ShapePictureJsonStringLoaded result ->
        case result of
          Ok "" -> { model | httpLoadResult = Err (Http.BadBody "Empty result for LoadShapePicture") }
          Ok s  -> updateModel (DecodeJson s) { model | httpLoadResult = result }
          Err e -> { model | httpLoadResult = result }
      LinkClicked urlRequest -> model
      UrlChanged  url        -> { model | url       = url
                                        , queryInfo = getQueryInfo url
                                }
      SavePicture pic        -> { model | picture = Just pic
                                        , saved   = []
                                }
      ImgLoaded  imgDs       -> { model | picture = Just (Picture (maybePictureToPicture model.picture).url imgDs.width imgDs.height) }
      DecodeJson jsonText ->
        case resultDecode jsonText of
            Err jsonError ->
                { model | jsonFile = { text  = model.jsonFile.text
                                     , error = Json.Decode.errorToString jsonError
                                     }
                }
            Ok pl ->
                { model | saved       = pl.shapes
                        , jsonFile    = { text  = jsonText
                                        , error = ""
                                        }
                        , picture     = pl.picture
                }
      LoadShapePicture    _ -> model
      ShapePicturePreview   -> { model | queryInfo = let queryInfo = model.queryInfo in { queryInfo | edit = False }
                                       , url       = let thisUrl   = model.url       in { thisUrl   | query = case thisUrl.query of
                                                                                                                Just q  -> Just ("edit=false&"++q)
                                                                                                                Nothing -> Just "edit=false"
                                                                                        }
                               }
updateCmdMsg : Msg -> Model -> Cmd Msg
updateCmdMsg msg model =
    case msg of
      ManageModal                  _ _ -> Cmd.none
      ManageShapes                 _ _ -> Cmd.none
      SvgShapeHover                _   -> Cmd.none
      UpdateMousePos               _   -> Cmd.none
      InputDrawShape                   -> Cmd.none
      OpenDrawShape                _   -> Cmd.none
      SavePicture                  _   -> Cmd.none
      ImgLoaded                    _   -> Cmd.none
      DecodeJson                   _   -> Cmd.none
      ShapePictureJsonStringLoaded _   -> Cmd.none
      LoadShapePicture             url -> Nav.load (Url.toString (let thisUrl = model.url in { thisUrl | query = Just ("url="++url) }))
      ShapePicturePreview              ->
        Nav.replaceUrl model.key (Url.toString (let thisUrl = model.url
                                                in {thisUrl | query = case thisUrl.query of
                                                                        Just q  -> Just ("edit=false&"++q)
                                                                        Nothing -> Just "edit=false"
                                                   }
                                               )
                                 )
      LinkClicked urlReq ->
        case urlReq of
          Browser.Internal url  -> Nav.pushUrl model.key (Url.toString url)
          Browser.External href -> Nav.load href
      UrlChanged     url -> Cmd.none
